#!/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# download-osec-vm-image 
# 
# download the initial OsecVM.ova image
#
# Copyright 2013-2014 X-Net and AIT Austrian Institute of Technology
# 
# 
#     X-Net Services GmbH
#     Elisabethstrasse 1
#     4020 Linz
#     AUSTRIA
#     https://www.x-net.at
# 
#     AIT Austrian Institute of Technology
#     Donau City Strasse 1
#     1220 Wien
#     AUSTRIA
#     http://www.ait.ac.at
# 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#    http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import errno
import os
import sys
import urllib2

from PyQt4 import QtCore
from PyQt4 import QtGui

# local
from environment import Environment


# ------------------------------------------------------------
# global vars


vm_path = os.path.normpath(os.path.join(sys.path[0], '..', 'vm'))


# ------------------------------------------------------------
# code


class DownloadDialog(QtGui.QDialog):
    
    """Download the initial OsecVM image dialog."""

    def __init__(self, parent = None, flags = QtCore.Qt.WindowFlags(0)):
        
        # super call and widget init
        super(DownloadDialog, self).__init__(parent, flags)
        self.setWindowTitle('Download initial OpenSecurity VM image')
        self.setup_ui()

        # setup download thread
        self._downloader = Downloader()
        self._downloader.downloaded.connect(self.downloader_update)
        self._thread = QtCore.QThread()
        self._downloader.moveToThread(self._thread)
        self._downloader.finished.connect(self._thread.quit)
        self._thread.started.connect(self._downloader.work)
        self._thread.finished.connect(self.finished_downloader)
        self._thread.start() 


    def clicked_cancel(self):

        """button cancel clicked"""

        # tell download thread to stop
        self._downloader.stop = True


    def downloader_update(self, current, total):
        
        """New download values present"""
        if self.pbDownload.minimum() != 0:
            self.pbDownload.setMinimum(0)
        if self.pbDownload.maximum() != total:
            self.pbDownload.setMaximum(total)
        self.pbDownload.setValue(current)


    def finished_downloader(self):
        
        """Download thread has finished (either success or abort)"""
        if not self._downloader.error is None:
            QtGui.QMessageBox.critical(None, 'OpenSecurity Download VM image error', self._downloader.error)

        self.accept();


    def setup_ui(self):
        
        """Create the widgets."""
        
        lyMain = QtGui.QVBoxLayout(self)
        lyMain.setContentsMargins(8, 8, 8, 8)

        lbTitle = QtGui.QLabel('Downloading OpenSecurity VM image')
        lyMain.addWidget(lbTitle)
        self.pbDownload = QtGui.QProgressBar()
        self.pbDownload.setTextVisible(True)
        lyMain.addWidget(self.pbDownload)
       
        # buttons
        lyButton = QtGui.QHBoxLayout()
        lyMain.addLayout(lyButton)
        
        lyButton.addStretch(1)
        btnCancel = QtGui.QPushButton('&Cancel', self)
        btnCancel.setMinimumWidth(100)
        lyButton.addWidget(btnCancel)
        
        # connectors
        btnCancel.clicked.connect(self.clicked_cancel)
        
        # reduce to the max
        self.setMinimumWidth(400)
        self.resize(lyMain.minimumSize())


class Downloader(QtCore.QObject):

    """This is the worker thread when downloading"""

    downloaded = QtCore.pyqtSignal(int, int)
    finished = QtCore.pyqtSignal()

    def __init__(self):
        
        # super call 
        super(Downloader, self).__init__()
        self.error = None
        self.stop = False

        # place to store the OsecVM image
        if not os.path.exists(vm_path):
            os.mkdir(vm_path)
 
       
    def work(self):

        """Thread method: download until finished or stopped"""

        url = "http://service.x-net.at/opensecurity/OsecVM_latest.ova"

        filename_download = os.path.join(vm_path, 'OsecVM_latest.ova')
        filename_target = os.path.join(vm_path, 'OsecVM.ova')

        # open URL
        try: 
            resp = urllib2.urlopen(url)
        except urllib2.URLError as e:
            self.error = 'URLError: ' + str(e.reason)
            self.finished.emit()
            return
        except urllib2.HTTPError as e:
            self.error = 'HTTPError: ' + str(e.reason)
            self.finished.emit()
            return

        # download a CHUNK of data and then update the GUI
        totalSize = int(resp.info().getheader('Content-Length').strip())
        currentSize = 0
        CHUNK_SIZE = 32768
        self.downloaded.emit(currentSize, totalSize)

        f = open(filename_download, 'w')
        while (not self.stop) and (currentSize < totalSize):
            data = resp.read(CHUNK_SIZE)
            currentSize += len(data)
            f.write(data)
            self.downloaded.emit(currentSize, totalSize)

        f.flush()
        f.close()

        # rename the download to OsecVM.ova
        if os.path.exists(filename_target):
            os.remove(filename_target)
        os.rename(filename_download, filename_target)

        self.finished.emit()


def main():

    """entry point"""
    
    # launch Qt
    global app
    app = QtGui.QApplication(sys.argv)

    # prebuild the pixmap cache: fetch all jpg, png and jpeg images and load them
    image_path = os.path.join(sys.path[0], '..', 'gfx')
    for file in os.listdir(image_path):
        if file.lower().rpartition('.')[2] in ('jpg', 'png', 'jpeg'):
            QtGui.QPixmapCache.insert(file.lower().rpartition('.')[0], QtGui.QPixmap(os.path.join(image_path, file)))
            
    # we should have now our application icon
    app.setWindowIcon(QtGui.QIcon(QtGui.QPixmapCache.find('opensecurity_icon_64')))

    # check essential permissions
    try:
        f = open(os.path.join(vm_path, '.x'), 'w')
        f.write('test')
        f.flush()
        f.close()
        os.remove(os.path.join(vm_path, '.x'))
    except IOError as e:
        if e.errno == errno.EACCES:
            QtGui.QMessageBox.critical(None, 'OpenSecurity Download VM image error', 'No write access to "' + str(vm_path) + '"\nIs this run as Administrator?')
            sys.exit(1)
        else:
            QtGui.QMessageBox.critical(None, 'OpenSecurity Download VM image error', 'Error occured.\nErrno: ' + str(e.errno))
            sys.exit(1)

    # open download dialog
    w = DownloadDialog()
    w.show()

    # on with the show
    sys.exit(app.exec_())
   
    
# start
if __name__ == "__main__":
    main()

